#!/usr/bin/env python  
# -*- coding:utf-8 _*-
""" 
@author: xl
@file: enums.py 
@time: 2021/08/09
@contact: 
@site:  
@software: PyCharm 
"""
import enum


class WarnningType(enum.Enum):
    urgent = "urgent"  # 紧急
    secondary = "secondary"  # 次要
    prompt = "prompt"  # 提示
    normal = "normal"


if __name__ == '__main__':
    print(WarnningType.normal.value)



#!/usr/bin/env python  
# -*- coding:utf-8 _*-
""" 
@author: xl
@file: conf.py 
@time: 2021/08/09
@contact: 
@site:  
@software: PyCharm 
"""
import os

import attrdict
from dotenv import load_dotenv
from loguru import logger

load_dotenv(verbose=True)

MQ_SERVER = os.environ.get('MQ_SERVER', '')
TOPIC = os.environ.get('TOPIC', '')
FEI_SHU = os.environ.get('FEI_SHU', '')

KAFKA_CONFIG = attrdict.AttrDict({
    'TOPIC': TOPIC,
    'KAFKA_SERVER': MQ_SERVER.strip().split('|')
})

CONF = attrdict.AttrDict({
    "KAFKA_CONFIG": KAFKA_CONFIG,
    "FEI_SHU": FEI_SHU
})

logger.info(CONF)



#!/usr/bin/env python  
# -*- coding:utf-8 _*-
""" 
@author: xl
@file: fastapi_hook.py 
@time: 2021/08/09
@contact: 
@site:  
@software: PyCharm 
"""
import asyncio
import traceback
import socket
from loguru import logger

from rosetta_service_monitor.client.service.mq_service import mq_serveice
from rosetta_service_monitor.client.background_task import backgound_task


def register_hook(app, project_name, **kwargs) -> None:
    """
    请求响应拦截 hook
    https://fastapi.tiangolo.com/tutorial/middleware/
    :param project_name:
    :param app:
    :return:
    """

    @app.middleware("http")
    async def monitor(request, call_next):
        path = request.url.path
        try:
            response = await call_next(request)
        except KeyboardInterrupt:
            raise KeyboardInterrupt
        except Exception as err:
            info = f'{err}:{err.__class__.__name__}\n{traceback.format_exc()}'
            request_data = {
                'path': path,
                'body': str(await request.body()),
                'query_params': str(request.query_params),
                'form': str(await request.form())
            }
            # logger.info(request_data)
            backgound_task(mq_serveice.send_mq, project_name=project_name, err_msg=info, trace_id='',
                           request=request_data, **kwargs)
            # mq_serveice.send_mq(project_name=project_name, err_msg=info, trace_id='', request=request_data, **kwargs)
            raise err
        return response



#!/usr/bin/env python  
# -*- coding:utf-8 _*-
""" 
@author: xl
@file: monitor.py 
@time: 2021/08/09
@contact: 
@site:  
@software: PyCharm 
"""
import asyncio
from functools import wraps
import traceback
import time

from loguru import logger

from rosetta_service_monitor.client.background_task import backgound_task
from rosetta_service_monitor.client.service.mq_service import mq_serveice


def service_monitor(project_name, **kwargs):
    logger.debug(f"Current Project is: {project_name}")
    service_kwargs = kwargs

    def monitor(func):
        if asyncio.iscoroutinefunction(func):
            @wraps(func)
            async def wrapper(*args, **kwargs):
                try:
                    start_time = time.time()
                    resp = await func(*args, **kwargs)
                    duration = time.time() - start_time
                    logger.info(f'{func.__name__} executed in {duration:.4f} s')
                    return resp
                except KeyboardInterrupt as e:
                    raise e
                except Exception as err:
                    logger.info(f"Get exception, send to mq...")
                    info = f'{err}:{err.__class__.__name__}, args:{args}, kwargs:{kwargs}\n{traceback.format_exc()}'
                    request_data = {}
                    backgound_task(mq_serveice.send_mq, project_name=project_name, err_msg=info, trace_id='',
                                   request=request_data, **service_kwargs)
                    raise err
        else:
            @wraps(func)
            def wrapper(*args, **kwargs):
                try:
                    start_time = time.time()
                    out = func(*args, **kwargs)
                    duration = time.time() - start_time
                    logger.info(f'{func.__name__} executed in {duration:.4f} s')
                    return out
                except KeyboardInterrupt as e:
                    raise e
                except Exception as err:
                    logger.info(f"Get exception, send to mq...")
                    info = f'{err}:{err.__class__.__name__}, args:{args}, kwargs:{kwargs}\n{traceback.format_exc()}'
                    request_data = {}
                    backgound_task(mq_serveice.send_mq, project_name=project_name, err_msg=info, trace_id='',
                                   request=request_data, **service_kwargs)
                    raise err

        return wrapper

    return monitor



#!/usr/bin/env python  
# -*- coding:utf-8 _*-
""" 
@author: xl
@file: mq_service.py 
@time: 2021/08/09
@contact: 
@site:  
@software: PyCharm 
"""
import asyncio
import json
import socket
import threading
from traceback import format_exc

from kafka import KafkaProducer
import os

from loguru import logger

from rosetta_service_monitor.client.conf import CONF
from rosetta_service_monitor.client.enums import WarnningType


class MQService:
    def __init__(self, conf):
        self._topic = conf.KAFKA_CONFIG.TOPIC
        self._server = conf.KAFKA_CONFIG.KAFKA_SERVER
        producer = KafkaProducer(
            bootstrap_servers=self._server,
            value_serializer=lambda v: json.dumps(v, ensure_ascii=False).encode("utf8"),
        )
        self.producer = producer

    async def send_mq(self, project_name: str, err_msg: str, trace_id: str, request: dict,
                      warnning_type: WarnningType = WarnningType.normal, **kwargs):
        hostname = socket.gethostname()
        host_info = {
            'hostname': hostname,
            'ip': os.getenv("HOST_IP") or socket.gethostbyname(hostname),
            'pod_name': os.getenv("MY_POD_NAME"),
            'pod_ip': os.getenv("POD_IP")
        }

        msg = {
            'project_name': project_name,
            'trace_id': trace_id,
            'reason': err_msg,
            'request': request,
            'hostinfo': host_info,
            'stage': warnning_type.value
        }
        msg.update(kwargs)
        future = self.producer.send(
            topic=self._topic,
            value=msg,
        )
        record_metadata = future.get(timeout=10)
        return record_metadata.topic

    def close(self):
        if self.producer is not None:
            self.producer.close()
            self.producer = None


mq_serveice = MQService(conf=CONF)



#!/usr/bin/env python  
# -*- coding:utf-8 _*-
""" 
@author: xl

@file: setup.py 
@time: 2021/05/06
@contact: 
@site:  
@software: PyCharm 
"""
from setuptools import setup, find_packages

python_requires = '>=3.7'
# or
# from distutils.core import setup
install_requires = []
print(find_packages(where='.', exclude=('tests', 'tests.*', 'test', 'test/*', 'backend'),
                    include=("rosetta_service_monitor", "rosetta_service_monitor.client")))

with open('requirements.txt', 'r') as f:
    for line in f.readlines():
        line = line.strip()
        if line and not line.startswith("#"): install_requires.append(line)

setup(
    name='rosetta_service_monitor',  # 包名字
    version='0.1.0',  # 包版本
    description='service monitor data collect client',  # 简单描述
    author='xialei',  # 作者
    author_email='xialei@joyy.sg',  # 作者邮箱
    url='',  # 包的主页
    packages=find_packages(where='.', exclude=('tests', 'tests.*', 'test', 'test/*', 'backend'),
                           include=("rosetta_service_monitor", "rosetta_service_monitor.client",
                                    "rosetta_service_monitor.client.*")),
    # 包
    # scripts=['entrypoint.sh'],
    # entry_points={"console_scripts": ["crawler-cli=backend.manager:cli"]},
    python_requires=python_requires,
    install_requires=install_requires,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



#!/usr/bin/env python  
# -*- coding:utf-8 _*-
""" 
@author: xl
@file: background_task.py 
@time: 2021/08/10
@contact: 
@site:  
@software: PyCharm 
"""

# 定义一个专门创建事件循环loop的函数，在另一个线程中启动它
import asyncio
import threading
from loguru import logger
from traceback import format_exc


def start_loop(loop):
    asyncio.set_event_loop(loop)
    loop.run_forever()


def backgound_task(func, *args, **kwargs):
    try:
        coroutine1 = func(*args, **kwargs)
        new_loop = asyncio.new_event_loop()  # 在当前线程下创建时间循环，（未启用），在start_loop里面启动它
        t = threading.Thread(target=start_loop, args=(new_loop,))  # 通过当前线程开启新的线程去启动事件循环
        t.start()

        asyncio.run_coroutine_threadsafe(coroutine1, new_loop)  # 这几个是关键，代表在新线程中事件循环不断“游走”执行
    except Exception as err:
        info = f'{err.__class__.__name__}:{err}\n{format_exc()}\n'
        logger.warning(info)
